#version 330 core

#define MAX_LIGHT_NUM 6

layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;

uniform mat4 projection;
uniform mat4 view;
uniform mat4 model;
uniform float worldScale;
uniform int vLightNum;
uniform vec3 LightInvDirection_worldspace[MAX_LIGHT_NUM];

out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace[MAX_LIGHT_NUM];

void main() 
{
	vec4 scaledPos;
	scaledPos.x = position.x * worldScale;
	scaledPos.y = position.y * worldScale;
	scaledPos.z = position.z * worldScale;
	scaledPos.w = 1.0f;

	gl_Position = projection * view * model * scaledPos;

	EyeDirection_cameraspace = vec3(0,0,0) - ( view * model * scaledPos).xyz;

	// Vector that goes from the vertex to the light, in camera space
	for(int lightIndex = 0; lightIndex < vLightNum; lightIndex++){
		LightDirection_cameraspace[lightIndex] = (view*vec4(LightInvDirection_worldspace[lightIndex],0)).xyz;	
	}

	// Normal of the the vertex, in camera space
	Normal_cameraspace = ( view * model * vec4(normal,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if n
}