#version 330 core

#define MAX_LIGHT_NUM 6

// Interpolated values from the vertex shaders
in vec2 UV;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace[MAX_LIGHT_NUM];

// Ouput data
layout(location = 0) out vec4 color;

// Values that stay constant for the whole mesh.
uniform int textureMode;
uniform sampler1D myTexture1D;
uniform sampler2D myTexture2D;
uniform mat4 MV;
uniform int fLightNum;
uniform vec3 LightPosition_worldspace;
uniform vec4 compColor;
uniform vec3 lightColor[MAX_LIGHT_NUM];
uniform float lightPower[MAX_LIGHT_NUM];

void calculateLight(int lightIndex, out float cosTheta, out float cosAlpha)
{
	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace[lightIndex] );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendiular to the triangle -> 0
	//  - light is behind the triangle -> 0
	cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	cosAlpha = clamp( dot( E,R ), 0,1 );
};

void main()
{
	// Material properties
	vec3 MaterialDiffuseColor;
	if(textureMode == 0){
		MaterialDiffuseColor = vec3(compColor);
	}
	else if(textureMode == 1){
		MaterialDiffuseColor = texture(myTexture1D, UV.x).rgb;
	}
	else{
		MaterialDiffuseColor = texture( myTexture2D, UV).rgb;
	}

	vec3 MaterialAmbientColor = vec3(0.2,0.2,0.2) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.2,0.2,0.2);

	vec3 outColor = MaterialAmbientColor;

	float cosTheta;
	float cosAlpha;

	for(int lightIndex = 0; lightIndex < fLightNum; lightIndex++)
	{	
		calculateLight(lightIndex,cosTheta,cosAlpha);

		outColor += 
		// Diffuse : "color" of the object
		MaterialDiffuseColor * lightColor[lightIndex] * lightPower[lightIndex] * cosTheta +
		// Specular : reflective highlight, like a mirror
		MaterialSpecularColor * lightColor[lightIndex] * lightPower[lightIndex] * pow(cosAlpha,5);
	}

	color = vec4(outColor,compColor.w);
}